#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind.utils
    ~~~~~~~~~~~

    :mod:``xmind.utils`` provide a handy way for internal used by
    xmind, and excepted that function defined here will be useful to
    others.

    :copyright:
    :license:
"""

__author__ = "aiqi@xmind.net <Woody Ai>"

import os
import time
import random
import tempfile
import zipfile

from hashlib import md5
from functools import wraps
from xml.dom.minidom import parse, parseString


temp_dir = tempfile.mkdtemp


def extract(path):
    return zipfile.ZipFile(path, "r")


def compress(path):
    return zipfile.ZipFile(path, "w")

########################             Path             #########################

join_path = os.path.join
split_ext = os.path.splitext


def get_abs_path(path):
    """
        Return the absolute path of a file

        If path contains a start point (eg Unix '/') then use the specified start point
        instead of the current working directory. The starting point of the file
        path is allowed to begin with a tilde "~", which will be replaced with the user's
        home directory.
    """

    fp, fn = os.path.split(path)
    if not fp:
        fp = os.getcwd()

    fp = os.path.abspath(os.path.expanduser(fp))

    return join_path(fp, fn)


#########################          Time         ###############################

def get_current_time():
    """
    Get the current time in milliseconds
    """
    return long(round(time.time() * 1000))


def readable_time(timestamp):
    """
    Convert timestamp to human-readable time format
    """
    # Timestamp in milliseconds, convert to seconds
    # Cause Python handle time in seconds
    timestampe_in_seconds = float(timestamp) / 1000
    return time.strftime(
        "%m/%d/%Y %H:%M:%S",
        time.gmtime(timestampe_in_seconds))


##########################           DOM            ###########################

parse_dom = parse
parse_dom_string = parseString
# def create_document():
#     return dom.Document()
#
#
# def create_element(tagName, namespaceURI=None, prefix=None, localName=None):
#     return dom.Element(tagName, namespaceURI, prefix, localName)
#
#
# def load_XML(stream):
#     """
#         Create new Document while occure load XML error
#     """
#     try:
#         return dom.parse(stream)
#     except:
#         return create_document()


##########################           Misc             #########################

def generate_id():
    """
    Generate unique 26-digit random string
    """
    # FIXME: Why not use something like the builtin uuid.uuid1() method?
    # md5 current time get 32-digit random string
    timestamp = md5(str(get_current_time())).hexdigest()
    lotter = md5(str(random.random())).hexdigest()  # :)

    id = timestamp[19:] + lotter[:13]

    return id.decode("utf8")


############################        Decorator       ###########################

def prevent(func):
    """
        Decorate func with this to prevent raising an Exception when
        an error is encountered
    """
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except:
            return

    return wrapper


def check(attr):
    def decorator(method):
        """
            Decorate method with this to check whether the object
            has an attribute with the given name.
        """
        @wraps(method)
        def wrapper(self, *args, **kwargs):
            if hasattr(self, attr):
                return method(self, *args, **kwargs)

            return None
        return wrapper
    return decorator


def main():
    pass

if __name__ == '__main__':
    main()



#-*- coding: utf-8 -*-
__author__  = "8034.com"
__date__    = "2018-10-22"

import sys
import os
import xmind
from xmind.core import workbook,saver
from xmind.core.topic import TopicElement
from openpyxl import Workbook
from openpyxl.compat import range
from openpyxl.utils import get_column_letter
from openpyxl import load_workbook

from xxmind.libs.parse import Parse

class XmindParse(Parse):
    source_xmind_file = None
    templet_excel_file = None
    target_excel_file = None

    def __init__(self, xmind, excel, templet):
        self.source_xmind_file = xmind
        self.templet_excel_file = templet
        self.target_excel_file = excel
        pass

    # def parse_xmind(self, xmind_file):
    #     w = xmind.load(xmind_file)
    #     s1=w.getPrimarySheet() # get the first sheet
    #     # print(s1.getTitle())
    #     r1=s1.getRootTopic() # get the root topic of this sheet
    #     topics_l01 = r1.getSubTopics() 

    #     case=dict(group="",title="",priority="",pestep="",step="",expect="",remarks="",upload=u"是")
    #     # print("\t%(group)s, %(title)s, %(priority)s, %(pestep)s, %(step)s, %(expect)s, %(remarks)s, %(upload)s"% case)
        
    #     for topic_case_group in topics_l01:
    #         topic_case_group_name = topic_case_group.getTitle()# case group 
    #         case["group"]=topic_case_group_name
    #         # print(topic_case_group_name)
    #         topics_case_title = topic_case_group.getSubTopics()
    #         if (not topics_case_title ):break
    #         for topic_case_title in topics_case_title:
    #             topic_case_title_name=topic_case_title.getTitle()  # case title
    #             case["title"]=topic_case_title_name
    #             count_case_num = 1
    #             topics_case_mainstep = topic_case_title.getSubTopics() 
    #             if (not topics_case_mainstep ):break
    #             for topic_case_mainstep in topics_case_mainstep:
    #                 topic_case_mainstep_name=topic_case_mainstep.getTitle() # case mainstep 
    #                 topic_case_pestep_name = u""
    #                 if topic_case_mainstep.getNotes():
    #                     topic_case_pestep_name=topic_case_mainstep.getNotes().getContent() # case pestep 
    #                 case["pestep"]=topic_case_pestep_name
    #                 topics_case_substep = topic_case_mainstep.getSubTopics() 
    #                 if (not topics_case_substep ):break
    #                 for topic_case_substep in topics_case_substep:
    #                     topic_case_substep_name = topic_case_substep.getTitle() # case substep
    #                     topic_case_priority_name = u"中"
    #                     if topic_case_substep.getMarkers():
    #                         topic_case_priority_name = topic_case_substep.getMarkers()[0].getMarkerId() # # case priority
    #                         try:
    #                             topic_case_priority_name = self.switch_priority(str(topic_case_priority_name).decode("utf-8")) 
    #                         except(AttributeError):
    #                             topic_case_priority_name = self.switch_priority(topic_case_priority_name)
    #                             pass
    #                     case["priority"]= topic_case_priority_name
    #                     case["step"]= self.fit_step(topic_case_mainstep_name,topic_case_substep_name)
    #                     topics_case_expect = topic_case_substep.getSubTopics() 
    #                     if (not topics_case_expect ):break
    #                     for topic_case_expect in topics_case_expect:
    #                         topic_case_expect_name=topic_case_expect.getTitle() # case expect 
    #                         case["expect"]=topic_case_expect_name
    #                         topics_case_remarks_name = u""
    #                         if topic_case_expect.getNotes():
    #                             topics_case_remarks_name = topic_case_expect.getNotes().getContent()
    #                         case["remarks"] = topics_case_remarks_name
    #                         topics_case_upload = topic_case_expect.getSubTopics()
    #                         if (topics_case_upload and topics_case_upload[0].getTitle()==u"不上传"): # case upload  
    #                             case["upload"]=u"否"
    #                         else:
    #                             case["upload"]=u"是"
    #                         entity_case=case.copy()
    #                         if (count_case_num > 1):
    #                             entity_case["title"] = self.fit_title(entity_case["title"], count_case_num)
    #                             pass
    #                         count_case_num = count_case_num+1
    #                         yield entity_case

    def main(self):
        if (not self.templet_excel_file):
            return False
        if (not self.source_xmind_file):
            return False
        if (not self.target_excel_file):
            return False
        # print(self.source_xmind_file)
        # 打开用例模板文件
        
        workbook_xlsx = load_workbook(filename= self.templet_excel_file)
        sheet = workbook_xlsx.get_sheet_by_name((workbook_xlsx.sheetnames)[0])
        # sheet = workbook_xlsx.index(0)
        last_row_num = sheet.max_row
        print(last_row_num)
        srows = last_row_num
        for case in self.parse_xmind(self.source_xmind_file):
            print("\t%(group)s, %(title)s, %(priority)s, %(pestep)s, %(step)s, %(expect)s, %(remarks)s, %(upload)s"% case)
            srows=self.addCasetoworkbook(case,sheet, srows+1)
        workbook_xlsx.save(self.target_excel_file)
        return u"xlsx success"

        

    def addCasetoworkbook(self, case, sheet, srows):
        case_sheet = sheet
        # 行, 起始为 1; 列, 起始为 0
        # 用例部分 从 第二行(nrows=2)开始, 第八列(ncols=7)结束
        #所属功能,用例标题,  级别,  前置步骤, 执行步骤,预期结果,备注说明,是否上传
        #导入     导入      默认中  默认无   导入     导入    默认空    导入
        case_sheet.cell(row=srows, column=1, value=case["group"])
        case_sheet.cell(row=srows, column=2, value=case["title"])
        case_sheet.cell(row=srows, column=3, value=case["priority"])
        case_sheet.cell(row=srows, column=4, value=case["pestep"])
        case_sheet.cell(row=srows, column=5, value=case["step"])
        case_sheet.cell(row=srows, column=6, value=case["expect"])
        case_sheet.cell(row=srows, column=7, value=case["remarks"])
        case_sheet.cell(row=srows, column=8, value=case["upload"])
        return srows

    # def switch_priority(self,xmind_priority):
    #     if (xmind_priority == "priority-1"):
    #         return u"高"
    #     elif (xmind_priority == "priority-2"): 
    #         return u"中"
    #     elif (xmind_priority == "priority-3"): 
    #         return u"低"
    #     else:
    #         return  u"中"
    
    # def fit_step(self, mainstep,substep):
    #     separator = "\n"+ "="*10+ "\n"
    #     full_step = mainstep + separator + substep
    #     return full_step

    # def fit_title(self, title, num):
    #     return title + u" 组用例-%d"%(num)
    # pass


    # def file_extension(self, path): 
    #     return os.path.splitext(path)[1] 

    pass


if __name__ =="__main__":
    # 第一个参数定义为目标文件名
    # def __init__(self, xmind, excel, templet):
    xmind_Text_content = u"C:/Users/Administrator/Desktop/编写用例/编写用例.xmind"
    excel_file_Text_content = u"C:/Users/Administrator/Desktop/编写用例/to.xls"
    templet_Text_content= u"d:\\CODE\\VScode\\workspace\\test01\\xmind2.0\\xxmind\\templet\\example.xls"

    xmindParse = XmindParse(xmind_Text_content,excel_file_Text_content,templet_Text_content)
    xmindParse.main()
    pass



#-*- coding: utf-8 -*-
__author__  = "8034.com"
__date__    = "2018-10-22"

import xmind
import os

class Parse(object):
    def parse_xmind(self, xmind_file):
        w = xmind.load(xmind_file)
        s1=w.getPrimarySheet() # get the first sheet
        # print(s1.getTitle())
        r1=s1.getRootTopic() # get the root topic of this sheet
        topics_l01 = r1.getSubTopics() 

        case=dict(group="",title="",priority="",pestep="",step="",expect="",remarks="",upload=u"是")
        # print("\t%(group)s, %(title)s, %(priority)s, %(pestep)s, %(step)s, %(expect)s, %(remarks)s, %(upload)s"% case)
        
        for topic_case_group in topics_l01:
            topic_case_group_name = topic_case_group.getTitle()# case group 
            case["group"]=topic_case_group_name
            # print(topic_case_group_name)
            topics_case_title = topic_case_group.getSubTopics()
            if (not topics_case_title ):break
            for topic_case_title in topics_case_title:
                topic_case_title_name=topic_case_title.getTitle()  # case title
                case["title"]=topic_case_title_name
                count_case_num = 1
                topics_case_mainstep = topic_case_title.getSubTopics() 
                if (not topics_case_mainstep ):break
                for topic_case_mainstep in topics_case_mainstep:
                    topic_case_mainstep_name=topic_case_mainstep.getTitle() # case mainstep 
                    topic_case_pestep_name = u""
                    if topic_case_mainstep.getNotes():
                        topic_case_pestep_name=topic_case_mainstep.getNotes().getContent() # case pestep 
                    case["pestep"]=topic_case_pestep_name
                    topics_case_substep = topic_case_mainstep.getSubTopics() 
                    if (not topics_case_substep ):break
                    for topic_case_substep in topics_case_substep:
                        topic_case_substep_name = topic_case_substep.getTitle() # case substep
                        topic_case_priority_name = u"中"
                        if topic_case_substep.getMarkers():
                            topic_case_priority_name = topic_case_substep.getMarkers()[0].getMarkerId() # # case priority
                            if topic_case_priority_name: 
                                print(type(topic_case_priority_name))
                                topic_case_priority_name = self.switch_priority(topic_case_priority_name)
                                pass 
                        case["priority"]= topic_case_priority_name
                        case["step"]= self.fit_step(topic_case_mainstep_name,topic_case_substep_name)
                        topics_case_expect = topic_case_substep.getSubTopics() 
                        if (not topics_case_expect ):break
                        for topic_case_expect in topics_case_expect:
                            topic_case_expect_name=topic_case_expect.getTitle() # case expect 
                            case["expect"]=topic_case_expect_name
                            topics_case_remarks_name = u""
                            if topic_case_expect.getNotes():
                                topics_case_remarks_name = topic_case_expect.getNotes().getContent()
                            case["remarks"] = topics_case_remarks_name
                            topics_case_upload = topic_case_expect.getSubTopics()
                            if (topics_case_upload and topics_case_upload[0].getTitle()==u"不上传"): # case upload  
                                case["upload"]=u"否"
                            else:
                                case["upload"]=u"是"
                            entity_case=case.copy()
                            if (count_case_num > 1):
                                entity_case["title"] ="%s: %s"%(self.fit_title(entity_case["title"], count_case_num), topic_case_substep_name) 
                                pass
                            count_case_num = count_case_num+1
                            yield entity_case
                            # print(json.dumps(entity_case).decode('unicode-escape'))

    def switch_priority(self,xmind_priority):
        if (xmind_priority == "priority-1"):
            return u"高"
        elif (xmind_priority == "priority-2"): 
            return u"中"
        elif (xmind_priority == "priority-3"): 
            return u"低"
        else:
            return  u"中"
    
    def fit_step(self, mainstep,substep):
        separator = "\n"+ "="*10+ "\n"
        full_step = mainstep + separator + substep
        return full_step

    def fit_title(self, title, num):
        return title + u" 组用例-%d"%(num)
    def file_extension(self, path): 
        return os.path.splitext(path)[1] 

    pass
    


#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind.core.saver
    ~~~~~~~~~~~~~~~~~

    :copyright:
    :license:

"""

__author__ = "aiqi@xmind.net <Woody Ai>"

import codecs

from . import const
from .. import utils


class WorkbookSaver(object):
    def __init__(self, workbook):
        """ Save `WorkbookDocument` as XMind file.

        :param workbook: `WorkbookDocument` object

        """
        self._workbook = workbook

    def _get_content(self):
        content_path = utils.join_path(utils.temp_dir(), const.CONTENT_XML)

        with codecs.open(content_path, "w", encoding="utf-8") as f:
            self._workbook.output(f)

        return content_path

    def save(self, path=None):
        """
        Save the workbook to the given path. If the path is not given, then
        will save to the path set in workbook.
        """
        path = path or self._workbook.get_path()

        if not path:
            raise Exception("Please specify a filename for the XMind file")

        path = utils.get_abs_path(path)

        file_name, ext = utils.split_ext(path)

        if ext != const.XMIND_EXT:
            raise Exception("XMind filenames require a '%s' extension" % const.XMIND_EXT)

        content = self._get_content()

        f=utils.compress(path)
        f.write(content, const.CONTENT_XML)


def main():
    pass

if __name__ == "__main__":
    main()



#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind.core.relationship
    ~~~~~~~~~~~~~~~~~~~


    :copyright:
    :license:
"""

__author__ = "aiqi@xmind.net <Woody Ai>"

from . import const

from .mixin import WorkbookMixinElement
from .topic import TopicElement
from .title import TitleElement


class RelationshipElement(WorkbookMixinElement):
    TAG_NAME = const.TAG_RELATIONSHIP

    def __init__(self, node=None, ownerWorkbook=None):
        super(RelationshipElement, self).__init__(node, ownerWorkbook)

        self.addIdAttribute(const.ATTR_ID)

    def _get_title(self):
        return self.getFirstChildNodeByTagName(const.TAG_TITLE)

    def _find_end_point(self, id):
        owner_workbook = self.getOwnerWorkbook()
        if owner_workbook is None:
            return

        end_point = owner_workbook.getElementById(id)
        if end_point is None:
            return

        if end_point.tagName == const.TAG_TOPIC:
            return TopicElement(end_point, owner_workbook)

    # FIXME: Convert the following to getter/setter

    def getEnd1ID(self):
        return self.getAttribute(const.ATTR_END1)

    def setEnd1ID(self, id):
        self.setAttribute(const.ATTR_END1, id)
        self.updateModifiedTime()

    def getEnd2ID(self):
        return self.getAttribute(const.ATTR_END2)

    def setEnd2ID(self, id):
        self.setAttribute(const.ATTR_END2, id)
        self.updateModifiedTime()

    def getEnd1(self, end1_id):
        return self._find_end_point(end1_id)

    def getEnd2(self, end2_id):
        return self._find_end_point(end2_id)

    def getTitle(self):
        title = self._get_title()
        if title:
            title = TitleElement(title, self.getOwnerWorkbook())
            return title.getTextContent()

    def setTitle(self, text):
        _title = self._get_title()
        title = TitleElement(_title, self.getOwnerWorkbook())
        title.setTextContent(text)

        if _title is None:
            self.appendChild(title)

        self.updateModifiedTime()


class RelationshipsElement(WorkbookMixinElement):
    TAG_NAME = const.TAG_RELATIONSHIPS

    def __init__(self, node=None, ownerWorkbook=None):
        super(RelationshipsElement, self).__init__(node, ownerWorkbook)


def main():
    pass

if __name__ == "__main__":
    main()



#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind.core.title
    ~~~~~~~~~~~~~~~

    :copyright:
    :license:

"""

__author__ = "aiqi@xmind.net <Woody Ai>"

from . import const

from .mixin import WorkbookMixinElement


class TitleElement(WorkbookMixinElement):
    TAG_NAME = const.TAG_TITLE

    def __init__(self, node=None, ownerWorkbook=None):
        super(TitleElement, self).__init__(node, ownerWorkbook)


def main():
    pass

if __name__ == '__main__':
    main()



#!/usr/env/bin python
#-*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name="xmind",
    version="0.1a.0",
    packages=find_packages(),


    install_requires=["distribute"],

    author="Woody Ai",
    author_email="aiqi@xmind.net",
    description="The offical XMind python SDK",
    license="MIT",
    keywords="XMind, SDK, mind mapping",
    url="https://github.com/xmindltd/xmind-sdk-python"
)



#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind.core.notes
    ~~~~~~~~~~~~~~~~

    :copyright:
    :license:

"""

__author__ = "aiqi@xmind.net <Woody Ai>"

from . import const

from .mixin import TopicMixinElement


class NotesElement(TopicMixinElement):
    TAG_NAME = const.TAG_NOTES

    def __init__(self, node=None, ownerTopic=None):
        super(NotesElement, self).__init__(node, ownerTopic)

    def getContent(self, format=const.PLAIN_FORMAT_NOTE):
        """ Get notes content

        :parma format:  specified returned content format, plain text
                        by default.
        """

        content = self.getFirstChildNodeByTagName(format)

        if not content:
            return

        if format is const.PLAIN_FORMAT_NOTE:
            content = PlainNotes(node=content, ownerTopic=self.getOwnerTopic())
        else:
            raise Exception("Only support plain text notes right now")

        return content.getTextContent()


class _NoteContentElement(TopicMixinElement):
    def __init__(self, node=None, ownerTopic=None):
        super(_NoteContentElement, self).__init__(node, ownerTopic)

    def getFormat(self):
        return self.getImplementation().tagName


class PlainNotes(_NoteContentElement):
    """ Plain text notes

    :param content: utf8 plain text.
    :param node:    `xml.dom.Element` object`
    :param ownerTopic:  `xmind.core.topic.TopicElement` object

    """

    TAG_NAME = const.PLAIN_FORMAT_NOTE

    def __init__(self, content=None, node=None, ownerTopic=None):
        super(PlainNotes, self).__init__(node, ownerTopic)
        if content is not None:
            self.setTextContent(content)

    def setContent(self, content):
        self.setTextContent(content)


def main():
    pass

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
__author__  = "8034.com"
__date__    = "2018-11-08"

import sys
print(sys.getdefaultencoding())
import os
from xxmind.__version__ import __VERSION__
FILE_PATH = os.path.dirname(os.path.realpath(__file__))



class Main(object):
    if sys.version[:1] > '2':
        from xxmind.ui3 import Application
    else:
        from xxmind.ui2 import Application
    app = Application()
    # 窗口标题:
    app.master.title("%s - %s"%('Xmind转为xls文件', __VERSION__))
    favicon_path = os.path.join(FILE_PATH, 'favicon.ico')
    app.master.iconbitmap(favicon_path)
    # 主消息循环:
    app.master.mainloop()
    pass


#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind.core
    ~~~~~~~~~~

    :copyright:
    :license:

"""

__author__ = "aiqi@xmind.net <Woody Ai>"


from xml.dom import minidom as DOM

from .. import utils


def create_document():
    """:cls: ``xml.dom.Document`` object constructor
    """
    return DOM.Document()


def create_element(tag_name, namespaceURI=None, prefix=None, localName=None):
    """:cls: ``xml.dom.Element`` object constructor
    """
    element = DOM.Element(tag_name, namespaceURI, prefix, localName)

#    if ":" in tag_name:
#        prefix, local_name = tag_name.split(":")
#    else:
#        local_name = tag_name.split(":")
#    element.prefix = prefix
#    element.localName = local_name

    return element


class Node(object):
    """
    All of components of XMind workbook subclass Node
    """
    def __init__(self, node):
        self._node = node

    def _equals(self, obj=None):
        """
        Compare the passed object with the current instance
        """
        if obj is None or not isinstance(obj, self.__class__):
            return False
        if obj == self:
            return True
        return self.getImplementation() == obj.getImplementation()

    def getImplementation(self):
        """
        Get DOM implementation of passed node. Provides an interface to
        manipulate the DOM directly
        """
        return self._node

    def getOwnerDocument(self):
        raise NotImplementedError("This method requires an implementation!")

    def setOwnerDocument(self, doc):
        raise NotImplementedError("This method requires an implementation!")

    def getLocalName(self, qualifiedName):
        index = qualifiedName.find(":")
        if index >= 0:
            return qualifiedName[index + 1:]
        else:
            return qualifiedName

    def getPrefix(self, qualifiedName):
        index = qualifiedName.find(":")
        if index >= 0:
            return qualifiedName[:index + 1]

    def appendChild(self, node):
        """
        Append passed node to the end of child node list of this node
        """
        node.setOwnerDocument(self.getOwnerDocument())

        node_impel = node.getImplementation()

        return self._node.appendChild(node_impel)

    def insertBefore(self, new_node, ref_node):
        """
        Insert new node before ref_node. Please notice that ref_node
        must be a child of this node.
        """
        new_node.setOwnerDocument(self.getOwnerDocument())

        new_node_imple = new_node.getImplementation()
        ref_node_imple = ref_node.getImplementation()

        return self._node.insertBefore(new_node_imple, ref_node_imple)

    def getChildNodesByTagName(self, tag_name):
        """
        Search for all children with specified tag name under passed DOM
        implementation, instead of all descendants
        """
        child_nodes = []
        for node in self._node.childNodes:
            if node.nodeType == node.TEXT_NODE:
                continue

            if node.tagName == tag_name:
                child_nodes.append(node)

        return child_nodes

    def getFirstChildNodeByTagName(self, tag_name):
        child_nodes = self.getChildNodesByTagName(tag_name)

        if len(child_nodes) >= 1:
            return child_nodes[0]

    def getParentNode(self):
        return self._node.parentNode

    def _isOrphanNode(self, node):
        if node is None:
            return True
        if node.nodeType == node.DOCUMENT_NODE:
            return False

        return self._isOrphanNode(node.parentNode)

    def isOrphanNode(self):
        return self._isOrphanNode(self._node)

    def iterChildNodesByTagName(self, tag_name):
        for node in self._node.childNodes:
            if node.nodeType == node.TEXT_NODE:
                continue

            if node.tagName == tag_name:
                yield node

    def removeChild(self, child_node):
        child_node = child_node.getImplementation()
        self._node.removeChild(child_node)

    def output(self, output_stream):
        return self._node.writexml(output_stream,
                                   addindent="",
                                   newl="",
                                   encoding="utf-8")


class Document(Node):
    def __init__(self, node=None):
        # FIXME: Should really call the base class
        #super(Document, self).__init__()
        self._node = node or self._documentConstructor()
        # self.arg = arg

    def _documentConstructor(self):
        return DOM.Document()

    @property
    def documentElement(self):
        """
        Get root element of passed DOM implementation for manipulate
        """
        return self._node.documentElement

    def getOwnerDocument(self):
        return self._node

    def createElement(self, tag_name):
        return self._node.createElement(tag_name)

    def setVersion(self, version):
        element = self.documentElement
        if element and not element.hasAttribute("version"):
            element.setAttribute("version", version)

    def replaceVersion(self, version):
        element = self.documentElement
        if element:
            element.setAttribute("version", version)

    def getElementById(self, id):
        return self._node.getElementById(id)


class Element(Node):
    TAG_NAME = ""

    def __init__(self, node=None):
        # FIXME: Should really call the base class
        #super(Element, self).__init__()
        self._node = node or self._elementConstructor(
            self.TAG_NAME.decode("utf8"))

    def _elementConstructor(self, tag_name,
                            namespaceURI=None,
                            prefix=None, localName=None):
        element = DOM.Element(tag_name, namespaceURI, prefix, localName)

        prefix = self.getPrefix(tag_name)
        localName = self.getLocalName(tag_name)

        element.prefix = prefix
        element.localName = localName

        return element

    def getOwnerDocument(self):
        return self._node.ownerDocument

    def setOwnerDocument(self, doc_imple):
        self._node.ownerDocument = doc_imple

    def setAttributeNS(self, namespace, attr):
        """
        Set attributes with namespace to DOM implementation.
        Please notice that namespace must be a namespace name and
        namespace value. Attr composed by namespceURI, localName and value.
        """
        namespace_name, namespace_value = namespace
        if not self._node.hasAttribute(namespace_name):
            self._node.setAttribute(namespace_name, namespace_value)

        namespaceURI, localName, value = attr
        if not self._node.hasAttributeNS(namespaceURI, localName):
            qualifiedName = "%s:%s" % (namespace_name, localName)
            self._node.setAttributeNS(namespaceURI, qualifiedName, value)

    def getAttribute(self, attr_name):
        """
        Get attribute with specified name. And allowed get attribute with
        specified name in ``prefix:localName`` format.
        """
        if not self._node.hasAttribute(attr_name):
            localName = self.getLocalName(attr_name)
            if localName != attr_name:
                return self.getAttribute(localName)
            return

        return self._node.getAttribute(attr_name)

    def setAttribute(self, attr_name, attr_value=None):
        """
        Set attribute to element. Please notice that if ``attr_value`` is
        None and attribute with specified ``attr_name`` is exist,
        attribute will be removed.
        """
        if attr_value is not None:
            self._node.setAttribute(attr_name,
                                    str(attr_value).decode("utf8"))
        elif self._node.hasAttribute(attr_name):
            self._node.removeAttribute(attr_name)

    def createElement(self, tag_name):
        """
        Create new element. But created element doesn't add to the child
        node list of this element, invoke :func: ``self.appendChild`` or :func:
        ``self.insertBefore`` to add created element to the child node list of
        this element.
        """
        pass

    def addIdAttribute(self, attr_name):
        if not self._node.hasAttribute(attr_name):
            id = utils.generate_id()
            self._node.setAttribute(attr_name, id)

            if self.getOwnerDocument():
                self._node.setIdAttribute(attr_name)

    def getIndex(self):
        parent = self.getParentNode()
        if parent:
            index = 0
            for node in parent.childNodes:
                if self._node is node:
                    return index
                index += 1

        return -1

    def getTextContent(self):
        text = []
        for node in self._node.childNodes:
            if node.nodeType == DOM.Node.TEXT_NODE:
                text.append(node.data)

        if not len(text) > 0:
            return

        text = "\n".join(text)
        return text

    def setTextContent(self, data):
        for node in self._node.childNodes:
            if node.nodeType == DOM.Node.TEXT_NODE:
                self._node.removeChild(node)

        text = DOM.Text()
        text.data = data.decode("utf8")

        self._node.appendChild(text)


def main():
    pass

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
__author__  = "8034.com"
__date__    = "2018-11-08"

import sys
import os
from tkinter import *
# import tkFileDialog for python2.X

# python3.X
from tkinter import messagebox, filedialog


# FILE_PATH = os.path.dirname(os.path.realpath(__file__))
FILE_PATH = os.path.dirname(os.path.realpath(__file__))

def file_extension(path): 
    return os.path.splitext(path)[1] 
def file_name(path): 
    return os.path.splitext(path)[0] 
    
class Application(Frame):

    def __init__(self, master=None):
        Frame.__init__(self, master)
        self.pack()
        self.createWidgets(master)

    def selectXmindFile(self):
        string_filename=""
        print((os.path.join(FILE_PATH, 'templet')))
        
        filenames = filedialog.askopenfilename(initialdir=(os.path.join(FILE_PATH, 'templet')))
        print(filenames)
        if len(filenames) != 0:
            string_filename =filenames
            text = u"您选择的文件是："+string_filename
        else:
            text = u"您没有选择任何文件"
        print(text)
        self.xmind_Text.delete(0.0,END)
        self.xmind_Text.insert(1.0,string_filename if string_filename else text)

        src_path = os.path.dirname(string_filename)
        excel_file = os.path.join(src_path, u"to.xls")
        excel_file=excel_file.replace("\\","/")
        self.excel_file_Text.delete(0.0,END)
        self.excel_file_Text.insert(1.0,excel_file)
        return string_filename

    def selectTempletFile(self):
        string_filename=""
        filenames = filedialog.askopenfilename(initialdir=(os.path.join(FILE_PATH, 'templet')))
        print(filenames)
        if len(filenames) != 0:
            string_filename =filenames
            text = u"您选择的模板是："+string_filename
        else:
            text = u"您没有选择任何模板，将继续使用默认模板"
        print(text)
        templet_extension = file_extension(string_filename)

        excel_file_Text_content=(self.excel_file_Text.get('1.0',END)).strip()
        excel_file_file_name = file_name(excel_file_Text_content)
        self.templet_Text.delete(0.0,END)
        self.templet_Text.insert(1.0,string_filename if string_filename else text)
        if templet_extension == ".xlsx":
            self.change_Button['state'] = 'disabled'
            self.change_toxlsx_Button['state'] = 'normal'
            self.excel_file_Text.delete(0.0,END)
            self.excel_file_Text.insert(1.0,excel_file_file_name+templet_extension)
        else:
            self.change_Button['state'] = 'normal'
            self.change_toxlsx_Button['state'] = 'disabled'
        return string_filename


    def doxmind2xls(self):
        xmind_Text_content=(self.xmind_Text.get('1.0',END)).strip()
        excel_file_Text_content=(self.excel_file_Text.get('1.0',END)).strip()
        templet_Text_content=(self.templet_Text.get('1.0',END)).strip()

        if(templet_Text_content == u"使用默认模板" or templet_Text_content == u"使用默认模板" or templet_Text_content== u"您没有选择任何模板，将继续使用默认模板" ):
            templet_Text_content = os.path.join(FILE_PATH, 'templet/example.xls')
            templet_Text_content = templet_Text_content.replace("\\","/")
            print(">>"*10)
            pass
        temp_houzhui = file_extension(templet_Text_content)
        result = "fail"
        if temp_houzhui == ".xlsx":
            from xxmind.libs.xmind2xlsx import XmindParse
            xmindParse = XmindParse(xmind_Text_content,excel_file_Text_content,templet_Text_content)
            result = xmindParse.main()
            pass
        else:
            from xxmind.libs.xmind2xls import XmindParse
            xmindParse = XmindParse(xmind_Text_content,excel_file_Text_content,templet_Text_content)
            result = xmindParse.main()
            pass

        self.sp05_Label.config(text=result)
        print("**"*10)
        return "ok"


    def createWidgets(self,master=None):

        self.frame_1 = Frame(master)
        self.xmind_Text = Text(self.frame_1,height="1",width="60")
        self.xmind_Text.pack(side=LEFT, expand=YES)
        self.xmind_Text.insert(INSERT,u"xmind Path")
        self.sp01_Label = Label(self.frame_1, text=u'<==', height="1",width="5")
        self.sp01_Label.pack(side=LEFT, expand=YES)
        self.select_file_button = Button(self.frame_1, text=u'选择文件', command = self.selectXmindFile )
        self.select_file_button.pack()
        self.frame_1.pack(side=TOP)

        self.frame_2 = Frame(master)
        self.excel_file_Text = Text(self.frame_2,height="1",width="60")
        self.excel_file_Text.pack(side=LEFT, expand=YES)
        self.excel_file_Text.insert(INSERT,u"目标路径")
        self.sp02_Label = Label(self.frame_2, text=u'<==', height="1",width="5")
        self.sp02_Label.pack(side=LEFT, expand=YES)
        self.excel_file_Label = Label(self.frame_2, text=u'生成xls ', height="1")
        self.excel_file_Label.pack()
        self.frame_2.pack(side=TOP)

        self.frame_3 = Frame(master)
        self.templet_Text = Text(self.frame_3,height="1",width="60")
        self.templet_Text.pack(side=LEFT, expand=YES)
        self.templet_Text.insert(INSERT,u"使用默认模板")
        self.sp03_Label = Label(self.frame_3, text=u'<==', height="1",width="5")
        self.sp03_Label.pack(side=LEFT, expand=YES)
        self.templet_file_button = Button(self.frame_3, text=u'选择模板', command = self.selectTempletFile )
        self.templet_file_button.pack()
        self.frame_3.pack(side=TOP)

        self.frame_4 = Frame(master)
        self.change_toxlsx_Button = Button(self.frame_4, text='Xmind转为xlsx',state='disabled',command=self.doxmind2xls)
        self.change_toxlsx_Button.pack(side=LEFT, expand=YES)
        self.sp04_Label = Label(self.frame_4, text=u' ', height="1",width="5")
        self.sp04_Label.pack(side=LEFT, expand=YES)
        self.change_Button = Button(self.frame_4, text='Xmind转为xls',state='disabled', command=self.doxmind2xls)
        self.change_Button.pack(side=LEFT, expand=YES)
        self.sp05_Label = Label(self.frame_4, text=u' ', height="1",width="5")
        self.sp05_Label.pack(side=LEFT, expand=YES)
        self.quitButton = Button(self.frame_4, text='Quit', command=self.quit)
        self.quitButton.pack(side=LEFT)
        self.frame_4.pack(side=TOP)

        self.frame_5 = Frame(master)
        self.sp05_Label = Label(self.frame_5, text=u' ', height="1",width="80")
        self.sp05_Label.pack(side=LEFT, expand=YES)
        self.frame_5.pack(side=TOP)



#-*- coding: utf-8 -*-
__author__  = "8034.com"
__date__    = "2018-10-22"

import sys
import os
import xmind
from xmind.core import workbook,saver
from xmind.core.topic import TopicElement
import json
import xlrd
import xlwt
from xlutils.copy import copy as xlscopy

from xxmind.libs.parse import Parse

class XmindParse(Parse):
    source_xmind_file = None
    templet_excel_file = None
    target_excel_file = None

    def __init__(self, xmind, excel, templet):
        self.source_xmind_file = xmind
        self.templet_excel_file = templet
        self.target_excel_file = excel
        pass

    # def parse_xmind(self, xmind_file):
    #     w = xmind.load(xmind_file)
    #     s1=w.getPrimarySheet() # get the first sheet
    #     # print(s1.getTitle())
    #     r1=s1.getRootTopic() # get the root topic of this sheet
    #     topics_l01 = r1.getSubTopics() 

    #     case=dict(group="",title="",priority="",pestep="",step="",expect="",remarks="",upload=u"是")
    #     # print("\t%(group)s, %(title)s, %(priority)s, %(pestep)s, %(step)s, %(expect)s, %(remarks)s, %(upload)s"% case)
        
    #     for topic_case_group in topics_l01:
    #         topic_case_group_name = topic_case_group.getTitle()# case group 
    #         case["group"]=topic_case_group_name
    #         # print(topic_case_group_name)
    #         topics_case_title = topic_case_group.getSubTopics()
    #         if (not topics_case_title ):break
    #         for topic_case_title in topics_case_title:
    #             topic_case_title_name=topic_case_title.getTitle()  # case title
    #             case["title"]=topic_case_title_name
    #             count_case_num = 1
    #             topics_case_mainstep = topic_case_title.getSubTopics() 
    #             if (not topics_case_mainstep ):break
    #             for topic_case_mainstep in topics_case_mainstep:
    #                 topic_case_mainstep_name=topic_case_mainstep.getTitle() # case mainstep 
    #                 topic_case_pestep_name = u""
    #                 if topic_case_mainstep.getNotes():
    #                     topic_case_pestep_name=topic_case_mainstep.getNotes().getContent() # case pestep 
    #                 case["pestep"]=topic_case_pestep_name
    #                 topics_case_substep = topic_case_mainstep.getSubTopics() 
    #                 if (not topics_case_substep ):break
    #                 for topic_case_substep in topics_case_substep:
    #                     topic_case_substep_name = topic_case_substep.getTitle() # case substep
    #                     topic_case_priority_name = u"中"
    #                     if topic_case_substep.getMarkers():
    #                         topic_case_priority_name = topic_case_substep.getMarkers()[0].getMarkerId() # # case priority
    #                         topic_case_priority_name = self.switch_priority(str(topic_case_priority_name).decode("utf-8")) 
    #                     case["priority"]= topic_case_priority_name
    #                     case["step"]= self.fit_step(topic_case_mainstep_name,topic_case_substep_name)
    #                     topics_case_expect = topic_case_substep.getSubTopics() 
    #                     if (not topics_case_expect ):break
    #                     for topic_case_expect in topics_case_expect:
    #                         topic_case_expect_name=topic_case_expect.getTitle() # case expect 
    #                         case["expect"]=topic_case_expect_name
    #                         topics_case_remarks_name = u""
    #                         if topic_case_expect.getNotes():
    #                             topics_case_remarks_name = topic_case_expect.getNotes().getContent()
    #                         case["remarks"] = topics_case_remarks_name
    #                         topics_case_upload = topic_case_expect.getSubTopics()
    #                         if (topics_case_upload and topics_case_upload[0].getTitle()==u"不上传"): # case upload  
    #                             case["upload"]=u"否"
    #                         else:
    #                             case["upload"]=u"是"
    #                         entity_case=case.copy()
    #                         if (count_case_num > 1):
    #                             entity_case["title"] = self.fit_title(entity_case["title"], count_case_num)
    #                             pass
    #                         count_case_num = count_case_num+1
    #                         yield entity_case
    #                         # print(json.dumps(entity_case).decode('unicode-escape'))

    def main(self):
        if (not self.templet_excel_file):
            return False
        if (not self.source_xmind_file):
            return False
        if (not self.target_excel_file):
            return False
        # print(self.source_xmind_file)
        # 打开用例模板文件
        workbook = xlrd.open_workbook(self.templet_excel_file, encoding_override='GB2312',formatting_info=True) 
        sheet = workbook.sheet_by_index(0)
        last_row_num = sheet.nrows
        # print(last_row_num)
        to_workbook = xlscopy(workbook)

        srows = last_row_num
        for case in self.parse_xmind(self.source_xmind_file):
            print("\t%(group)s, %(title)s, %(priority)s, %(pestep)s, %(step)s, %(expect)s, %(remarks)s, %(upload)s"% case)
            srows=self.addCasetoworkbook(case,to_workbook, srows)
        to_workbook.save(self.target_excel_file)
        return u"xls success"

        

    def addCasetoworkbook(self, case, to_workbook, srows):
        case_sheet = to_workbook.get_sheet(0) # 行, 起始为 1; 列, 起始为 0
        # 用例部分 从 第二行(nrows=2)开始, 第八列(ncols=7)结束
        #所属功能,用例标题,  级别,  前置步骤, 执行步骤,预期结果,备注说明,是否上传
        #导入     导入      默认中  默认无   导入     导入    默认空    导入
        case_sheet.write(srows,0,case["group"])
        case_sheet.write(srows,1,case["title"]) 
        case_sheet.write(srows,2,case["priority"])
        case_sheet.write(srows,3,case["pestep"])
        case_sheet.write(srows,4,case["step"])
        case_sheet.write(srows,5,case["expect"])
        case_sheet.write(srows,6,case["remarks"])
        case_sheet.write(srows,7,case["upload"])
        return srows+1

    # def switch_priority(self,xmind_priority):
    #     if (xmind_priority == "priority-1"):
    #         return u"高"
    #     elif (xmind_priority == "priority-2"): 
    #         return u"中"
    #     elif (xmind_priority == "priority-3"): 
    #         return u"低"
    #     else:
    #         return  u"中"
    
    # def fit_step(self, mainstep,substep):
    #     separator = "\n"+ "="*10+ "\n"
    #     full_step = mainstep + separator + substep
    #     return full_step

    # def fit_title(self, title, num):
    #     return title + u" 组用例-%d"%(num)
    pass




if __name__ =="__main__":
    # 第一个参数定义为目标文件名
    # def __init__(self, xmind, excel, templet):
    xmind_Text_content = u"C:/Users/Administrator/Desktop/编写用例/编写用例.xmind"
    excel_file_Text_content = u"C:/Users/Administrator/Desktop/编写用例/to.xls"
    templet_Text_content= u"d:\\CODE\\VScode\\workspace\\test01\\xmind2.0\\xxmind\\templet\\example.xls"

    xmindParse = XmindParse(xmind_Text_content,excel_file_Text_content,templet_Text_content)
    xmindParse.main()
    pass



#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind
    ~~~~~

    :copyright:
    :license:

"""


__version__ = "0.1a.0"
__author__ = "aiqi@xmind.net <Woody Ai>"

from xmind.core.loader import WorkbookLoader
from xmind.core.saver import WorkbookSaver


def load(path):
    """ Load XMind workbook from given path. If file no
    exist on given path then created new one.

    """
    loader = WorkbookLoader(path)
    return loader.get_workbook()


def save(workbook, path=None):
    """ Save workbook to given path. If path not given, then
    will save to path that set to workbook.

    """
    saver = WorkbookSaver(workbook)
    saver.save(path)


def main():
    pass

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind.core.const
    ~~~~~~~~~~~~~~~~

    :copyright:
    :license:
"""

__author__ = "aiqi@xmind.net <Woody Ai>"

XMIND_EXT = ".xmind"

VERSION = "2.0"
NAMESPACE = "xmlns"
XMAP = "urn:xmind:xmap:xmlns:content:2.0"

ATTACHMENTS_DIR = "attachments/"
MARKERS_DIR = "markers/"
META_INF_DIR = "META-INF/"
REVISIONS_DIR = "Revisions/"

CONTENT_XML = "content.xml"
STYLES_XML = "styles.xml"
META_XML = "meta.xml"
MANIFEST_XML = "META-INF/manifest.xml"
MARKER_SHEET_XML = "markerSheet.xml"
MARKER_SHEET = MARKERS_DIR + MARKER_SHEET_XML
REVISIONS_XML = "revisions.xml"

TAG_WORKBOOK = "xmap-content"
TAG_TOPIC = "topic"
TAG_TOPICS = "topics"
TAG_SHEET = "sheet"
TAG_TITLE = "title"
TAG_POSITION = "position"
TAG_CHILDREN = "children"
TAG_NOTES = "notes"
TAG_RELATIONSHIP = "relationship"
TAG_RELATIONSHIPS = "relationships"
TAG_MARKERREFS = "marker-refs"
TAG_MARKERREF  = "marker-ref"
ATTR_VERSION = "version"
ATTR_ID = "id"
ATTR_STYLE_ID = "style-id"
ATTR_TIMESTAMP = "timestamp"
ATTR_THEME = "theme"
ATTR_X = "svg:x"
ATTR_Y = "svg:y"
ATTR_HREF = "xlink:href"
ATTR_BRANCH = "branch"
ATTR_TYPE = "type"
ATTR_END1 = "end1"
ATTR_END2 = "end2"
ATTR_MARKERID = "marker-id"

NS_URI = "http://www.w3.org/1999/xhtml"

NS_XHTML = (NS_URI, "xhtml", "http://www.w3.org/1999/xhtml")
NS_XLINK = (NS_URI, "xlink", "http://www.w3.org/1999/xlink")
NS_SVG = (NS_URI, "svg", "http://www.w3.org/2000/svg")
NS_FO = (NS_URI, "fo", "http://www.w3.org/1999/XSL/Format")

VAL_FOLDED = "folded"

TOPIC_ROOT = "root"
TOPIC_ATTACHED = "attached"

FILE_PROTOCOL = "file://"
TOPIC_PROTOCOL = "xmind:#"
HTTP_PROTOCOL = "http://"
HTTPS_PROTOCOL = "https://"

HTML_FORMAT_NOTE = "html"
PLAIN_FORMAT_NOTE = "plain"



#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind.core.markerref
    ~~~~~~~~~~~~~~~

    :copyright:
    :license:

"""

__author__ = "stanypub@gmail.com <Stany MARCEL>"

from . import const
from .mixin import WorkbookMixinElement


class MarkerId:
    def __init__(self, name):
        self.name = name

    def __str__(self):
        return self.name

    def __repr__(self):
        return "<MarkerId: %s>" % self

    def getFamilly(self):
        return self.name.split('-')[0]


MarkerId.starRed         = 'star-red'
MarkerId.starOrange      = 'star-orange'
MarkerId.starYellow      = 'star-yellow'
MarkerId.starBlue        = 'star-blue'
MarkerId.starGreen       = 'star-green'
MarkerId.starPurple      = 'star-purple'

MarkerId.priority1       = 'priority-1'
MarkerId.priority2       = 'priority-2'
MarkerId.priority3       = 'priority-3'
MarkerId.priority4       = 'priority-4'
MarkerId.priority5       = 'priority-5'
MarkerId.priority6       = 'priority-6'
MarkerId.priority7       = 'priority-7'
MarkerId.priority8       = 'priority-8'
MarkerId.priority9       = 'priority-9'

MarkerId.smileySmile     = 'smiley-smile'
MarkerId.smileyLaugh     = 'smiley-laugh'
MarkerId.smileyAngry     = 'smiley-angry'
MarkerId.smileyCry       = 'smiley-cry'
MarkerId.smileySurprise  = 'smiley-surprise'
MarkerId.smileyBoring    = 'smiley-boring'

MarkerId.task0_8         = 'task-start'
MarkerId.task1_8         = 'task-oct'
MarkerId.task2_8         = 'task-quarter'
MarkerId.task3_8         = 'task-3oct'
MarkerId.task4_8         = 'task-half'
MarkerId.task5_8         = 'task-5oct'
MarkerId.task6_8         = 'task-3quar'
MarkerId.task7_8         = 'task-7oct'
MarkerId.task8_8         = 'task-done'

MarkerId.flagRed         = 'flag-red'
MarkerId.flagOrange      = 'flag-orange'
MarkerId.flagYellow      = 'flag-yellow'
MarkerId.flagBlue        = 'flag-blue'
MarkerId.flagGreen       = 'flag-green'
MarkerId.flagPurple      = 'flag-purple'

MarkerId.peopleRed       = 'people-red'
MarkerId.peopleOrange    = 'people-orange'
MarkerId.peopleYellow    = 'people-yellow'
MarkerId.peopleBlue      = 'people-blue'
MarkerId.peopleGreen     = 'people-green'
MarkerId.peoplePurple    = 'people-purple'

MarkerId.arrowUp         = 'arrow-up'
MarkerId.arrowUpRight    = 'arrow-up-right'
MarkerId.arrowRight      = 'arrow-right'
MarkerId.arrowDownRight  = 'arrow-down-right'
MarkerId.arrowDown       = 'arrow-down'
MarkerId.arrowDownLeft   = 'arrow-down-left'
MarkerId.arrowLeft       = 'arrow-left'
MarkerId.arrowUpLeft     = 'arrow-up-left'
MarkerId.arrowRefresh    = 'arrow-refresh'

MarkerId.symbolPlus      = 'symbol-plus'
MarkerId.symbolMinus     = 'symbol-minus'
MarkerId.symbolQuestion  = 'symbol-question'
MarkerId.symbolExclam    = 'symbol-exclam'
MarkerId.symbolInfo      = 'symbol-info'
MarkerId.symbolWrong     = 'symbol-wrong'
MarkerId.symbolRight     = 'symbol-right'

MarkerId.monthJan        = 'month-jan'
MarkerId.monthFeb        = 'month-feb'
MarkerId.monthMar        = 'month-mar'
MarkerId.monthApr        = 'month-apr'
MarkerId.monthMay        = 'month-may'
MarkerId.monthJun        = 'month-jun'
MarkerId.monthJul        = 'month-jul'
MarkerId.monthAug        = 'month-aug'
MarkerId.monthSep        = 'month-sep'
MarkerId.monthOct        = 'month-oct'
MarkerId.monthNov        = 'month-nov'
MarkerId.monthDec        = 'month-dec'

MarkerId.weekSun         = 'week-sun'
MarkerId.weekMon         = 'week-mon'
MarkerId.weekTue         = 'week-tue'
MarkerId.weekWed         = 'week-wed'
MarkerId.weekThu         = 'week-thu'
MarkerId.weekFri         = 'week-fri'
MarkerId.weekSat         = 'week-sat'


class MarkerRefsElement(WorkbookMixinElement):
    TAG_NAME = const.TAG_MARKERREFS

    def __init__(self, node=None, ownerWorkbook=None):
        super(MarkerRefsElement, self).__init__(node, ownerWorkbook)

class MarkerRefElement(WorkbookMixinElement):
    TAG_NAME = const.TAG_MARKERREF

    def __init__(self, node=None, ownerWorkbook=None):
        super(MarkerRefElement, self).__init__(node, ownerWorkbook)

    def getMarkerId(self):
        return MarkerId(self.getAttribute(const.ATTR_MARKERID))

    def setMarkerId(self, val):
        self.setAttribute(const.ATTR_MARKERID, str(val))

def main():
    pass

if __name__ == '__main__':
    main()



#-*- coding: utf-8 -*-
import xmind
from xmind.core import workbook,saver
from xmind.core.topic import TopicElement

w = xmind.load("test.xmind") # load an existing file or create a new workbook if nothing is found

s1=w.getPrimarySheet() # get the first sheet
s1.setTitle("first sheet") # set its title
r1=s1.getRootTopic() # get the root topic of this sheet
r1.setTitle("we don't care of this sheet") # set its title

s2=w.createSheet() # create a new sheet
s2.setTitle("second sheet")
r2=s2.getRootTopic()
r2.setTitle("root node")


t1=TopicElement() # create a new element
t1.setTopicHyperlink(s1.getID()) # set a link from this topic to the first sheet given by s1.getID()
t1.setTitle("redirection to the first sheet") # set its title

t2=TopicElement()
t2.setTitle("second node")
t2.setURLHyperlink("https://xmind.net") # set an hyperlink

t3=TopicElement()
t3.setTitle("third node")
t3.setPlainNotes("notes for this topic") # set notes (F4 in XMind)
t3.setTitle("topic with \n notes")

t4=TopicElement()
t4.setFileHyperlink("logo.jpeg") # set a file hyperlink
t4.setTitle("topic with a file")


# then the topics must be added to the root element

r2.addSubTopic(t1)
r2.addSubTopic(t2)
r2.addSubTopic(t3)
r2.addSubTopic(t4)

topics=r2.getSubTopics() # to loop on the subTopics
for topic in topics:
    topic.addMarker("yes")

w.addSheet(s2) # the second sheet is now added to the workbook
rel=s2.createRelationship(t1.getID(),t2.getID(),"test") # create a relationship
s2.addRelationship(rel) # and add to the sheet

xmind.save(w,"test2.xmind") # and we save


#-*- coding: utf-8 -*-
import xmind
from xmind.core import workbook,saver
from xmind.core.topic import TopicElement

w = xmind.load("test.xmind") # load an existing file or create a new workbook if nothing is found
w = xmind.load(u"mac.xmind")
s1=w.getPrimarySheet() # get the first sheet
# print(s1.getTitle())
r1=s1.getRootTopic() # get the root topic of this sheet
topics_l01 = r1.getSubTopics() 
for topic_case_group in topics_l01:
    topic_case_priority_name = topic_case_group.getMarkers()[0].getMarkerId() # # case priority
    print(topic_case_priority_name)
    pass


#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind.core.workbook
    ~~~~~~~~~~~~~~~~~~~

    :mod:``xmind.core.workbook`` implements the command XMind
    manipulations.

    :copyright:
    :license:
"""

__author__ = "aiqi@xmind.net <Woody Ai>"


from . import Document
from . import const

from .mixin import WorkbookMixinElement
from .sheet import SheetElement
from .topic import TopicElement
from .relationship import RelationshipElement

from .. import utils


class WorkbookElement(WorkbookMixinElement):
    TAG_NAME = const.TAG_WORKBOOK

    def __init__(self, node=None, ownerWorkbook=None):
        super(WorkbookElement, self).__init__(node, ownerWorkbook)

        # Initialize WorkbookElement with default attribute
        namespace = (const.NAMESPACE, const.XMAP)
        attrs = [const.NS_FO, const.NS_XHTML, const.NS_XLINK, const.NS_SVG]

        for attr in attrs:
            self.setAttributeNS(namespace, attr)

        # Initialize WorkbookElement need contains at least one SheetElement
        if not self.getSheets():
            sheet = self.createSheet()
            self.addSheet(sheet)

    def setOwnerWorkbook(self, workbook):
        raise Exception(
            """WorkbookDocument allowed only contains one WorkbookElement
            """)

    def getSheets(self):
        sheets = self.getChildNodesByTagName(const.TAG_SHEET)
        owner_workbook = self.getOwnerWorkbook()
        sheets = [SheetElement(sheet, owner_workbook) for sheet in sheets]

        return sheets

    def getSheetByIndex(self, index):
        sheets = self.getSheets()

        if index < 0 or index >= len(sheets):
            return

        return sheets[index]

    def createSheet(self):
        sheet = SheetElement(None, self.getOwnerWorkbook())
        return sheet

    def addSheet(self, sheet, index=None):
        sheets = self.getSheets()
        if index < 0 or index >= len(sheets):
            self.appendChild(sheet)
        else:
            self.insertBefore(sheet, sheets[index])

        self.updateModifiedTime()

    def removeSheet(self, sheet):
        sheets = self.getSheets()
        if len(sheets) <= 1:
            return

        if sheet.getParentNode() == self.getImplementation():
            self.removeChild(sheet)
            self.updateModifiedTime()

    def moveSheet(self, original_index, target_index):
        if original_index < 0 or original_index == target_index:
            return

        sheets = self.getSheets()
        if original_index >= len(sheets):
            return

        sheet = sheets[original_index]
        if not target_index < 0 and target_index < len(sheets) - 1:
            if original_index < target_index:
                target_index += 1
            else:
                target_index = target_index

            target = sheets[target_index]
            if target != sheet:
                self.removeChild(sheet)
                self.insertBefore(sheet, target)
        else:
            self.removeChild(sheet)
            self.appendChild(sheet)

        self.updateModifiedTime()

    def getVersion(self):
        return self.getAttribution(const.ATTR_VERSION)


class WorkbookDocument(Document):
    """ `WorkbookDocument` as central object correspond XMind workbook.
    """
    def __init__(self, node=None, path=None):
        """
        Construct new `WorkbookDocument` object

        :param node:    pass DOM node object and parse as `WorkbookDocument`
                        object. if node not given then created new one.

        :param path:    set workbook will to be placed.

        """
        super(WorkbookDocument, self).__init__(node)
        self._path = path
        # Initialize WorkbookDocument to make sure that contains
        # WorkbookElement as root.
        _workbook_element = self.getFirstChildNodeByTagName(
            const.TAG_WORKBOOK)

        self._workbook_element = WorkbookElement(
            _workbook_element,
            self)

        if not _workbook_element:
            self.appendChild(self._workbook_element)

        self.setVersion(const.VERSION)

    def getWorkbookElement(self):
        return self._workbook_element

    def _create_relationship(self):
        return RelationshipElement(None, self)

    def createRelationship(self, end1, end2):
        """ Create relationship with two topics. Pass two
        `TopicElement` object and retuen `RelationshipElement` object
        """
        sheet1 = end1.getOwnerSheet()
        sheet2 = end2.getOwnerSheet()

        if sheet1 and sheet2:
            if sheet1.getImplementation() == sheet2.getImplementation():

                rel = self._create_relationship()
                rel.setEnd1ID(end1.getID())
                rel.setEnd2ID(end2.getID())

                sheet1.addRelationships(rel)

                return rel

        raise Exception("Topics not on the same sheet!")

    def createTopic(self):
        """
        Create new `TopicElement` object and return. Please notice that
        this topic will not be added to the workbook.
        """
        return TopicElement(None, self)

    def getSheets(self):
        """
        List all sheets under workbook, if not sheets then return
        empty list
        """
        return self._workbook_element.getSheets()

    def getPrimarySheet(self):
        """
        Get the first sheet under workbook.
        """
        return self._workbook_element.getSheetByIndex(0)

    def createSheet(self):
        """
        Create new sheet. But please notice the new created sheet
        hasn't been added to the workbook. Invoke :method addSheet: to do that.
        """
        return self._workbook_element.createSheet()

    def addSheet(self, sheet, index=None):
        """
        Add a sheet to the workbook.

        :param sheet:   add passed `SheetElement` object to workbook.

        :param index:   insert sheet before another sheet that given by
                        index. If index not given, append sheet to the
                        sheets list.
        """
        self._workbook_element.addSheet(sheet, index)

    def removeSheet(self, sheet):
        """
        Remove a sheet from the workbook

        :param sheet:   remove passed `SheetElement` object
        """
        self._workbook_element.removeSheet(sheet)

    def moveSheet(self, original_index, target_index):
        """
        Move a sheet from the original index to the target index

        :param original_index:  index of the sheet will be moved.
                                `original_index` must be positive integer and
                                less than `target_index`.
        :param target_index:    index that sheet want to move to.
                                `target_index` must be positive integer and
                                less than the length of sheets list.
        """
        self._workbook_element.moveSheet(original_index, target_index)

    def getVersion(self):
        return self._workbook_element.getVersion()

    def getModifiedTime(self):
        return self._workbook_element.getModifiedTime()

    def updateModifiedTime(self):
        return self._workbook_element.updateModifiedTime()

    def setModifiedTime(self):
        return self._workbook_element.setModifiedTime()

    def get_path(self):
        if self._path:
            return utils.get_abs_path(self._path)

    def set_path(self, path):
        self._path = utils.get_abs_path(path)


def main():
    pass

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind.core.position
    ~~~~~~~~~~~~~~~~

    :copyright:
    :license:

"""

__author__ = "aiqi@xmind.net <Woody Ai>"

from . import const

from .mixin import WorkbookMixinElement


class PositionElement(WorkbookMixinElement):
    TAG_NAME = const.TAG_POSITION

    def __init__(self, node=None, ownerWorkbook=None):
        super(PositionElement, self).__init__(node, ownerWorkbook)

    # FIXME: These should be converted to getter/setters

    def getX(self):
        return self.getAttribute(const.ATTR_X)

    def getY(self):
        return self.getAttribute(const.ATTR_Y)

    def setX(self, x):
        self.setAttribute(const.ATTR_X, int(x))

    def setY(self, y):
        self.setAttribute(const.ATTR_Y, int(y))


def main():
    pass

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind.core.loader
    ~~~~~~~~~~~~~~~~~

    :copyright:
    :license:

"""

__author__ = "aiqi@xmind.net <Woody Ai>"

from . import const
from .workbook import WorkbookDocument

from .. import utils


class WorkbookLoader(object):
    def __init__(self, path):
        """ Load XMind workbook from given path

        :param path:    path to XMind file. If not an existing file,
                        will not raise an exception.

        """
        super(WorkbookLoader, self).__init__()
        self._input_source = utils.get_abs_path(path)

        file_name, ext = utils.split_ext(self._input_source)

        if ext != const.XMIND_EXT:
            raise Exception("The XMind filename is missing the '%s' extension!" % const.XMIND_EXT)

        # Input Stream
        self._content_stream = None

        try:
            with utils.extract(self._input_source) as input_stream:
                for stream in input_stream.namelist():
                    if stream == const.CONTENT_XML:
                        self._content_stream = utils.parse_dom_string(
                            input_stream.read(stream))
        except:
            pass

    def get_workbook(self):
        """ Parse XMind file to `WorkbookDocument` object and return
        """
        content = self._content_stream
        path = self._input_source

        workbook = WorkbookDocument(content, path)
        return workbook


def main():
    pass

if __name__ == "__main__":
    main()



#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind.core.topic
    ~~~~~~~~~~~~~~~~

    :copyright:
    :license:

"""

__author__ = "aiqi@xmind.net <Woody Ai>"

from . import const

from .mixin import WorkbookMixinElement
from .title import TitleElement
from .position import PositionElement
from .notes import NotesElement, PlainNotes
from .markerref import MarkerRefElement
from .markerref import MarkerRefsElement
from .markerref import MarkerId

from .. import utils


def split_hyperlink(hyperlink):
    colon = hyperlink.find(":")
    if colon < 0:
        protocol = None
    else:
        protocol = hyperlink[:colon]

    hyperlink = hyperlink[colon + 1:]
    while hyperlink.startswith("/"):
        hyperlink = hyperlink[1:]

    return (protocol, hyperlink)


class TopicElement(WorkbookMixinElement):
    TAG_NAME = const.TAG_TOPIC

    def __init__(self, node=None, ownerWorkbook=None):
        super(TopicElement, self).__init__(node, ownerWorkbook)

        self.addIdAttribute(const.ATTR_ID)

    def _get_title(self):
        return self.getFirstChildNodeByTagName(const.TAG_TITLE)

    def _get_markerrefs(self):
        return self.getFirstChildNodeByTagName(const.TAG_MARKERREFS)

    def _get_position(self):
        return self.getFirstChildNodeByTagName(const.TAG_POSITION)

    def _get_children(self):
        return self.getFirstChildNodeByTagName(const.TAG_CHILDREN)

    def _set_hyperlink(self, hyperlink):
        self.setAttribute(const.ATTR_HREF, hyperlink)
        #self.updateModifiedTime()

    def getOwnerSheet(self):
        parent = self.getParentNode()

        while parent and parent.tagName != const.TAG_SHEET:
            parent = parent.parentNode

        if not parent:
            return

        owner_workbook = self.getOwnerWorkbook()
        if not owner_workbook:
            return

        for sheet in owner_workbook.getSheets():
            if parent is sheet.getImplementation():
                return sheet

    def getTitle(self):
        title = self._get_title()
        if title:
            title = TitleElement(title, self.getOwnerWorkbook())
            return title.getTextContent()

    def setTitle(self, text):
        _title = self._get_title()
        title = TitleElement(_title, self.getOwnerWorkbook())
        title.setTextContent(text)

        if _title is None:
            self.appendChild(title)

        # self.updateModifiedTime()

    def getMarkers(self):
        refs = self._get_markerrefs()
        if not refs:
            return None
        tmp = MarkerRefsElement(refs, self.getOwnerWorkbook())
        markers = tmp.getChildNodesByTagName(const.TAG_MARKERREF)
        marker_list = []
        if markers:
            for i in markers:
                marker_list.append(MarkerRefElement(i, self.getOwnerWorkbook()))
        return marker_list

    def addMarker(self, markerId):

        if not markerId:
            return None
        if type(markerId) == str:
            markerId = MarkerId(markerId)

        refs = self._get_markerrefs()
        if not refs:
            tmp = MarkerRefsElement(None, self.getOwnerWorkbook())
            self.appendChild(tmp)
        else:
            tmp = MarkerRefsElement(refs, self.getOwnerWorkbook())
        markers = tmp.getChildNodesByTagName(const.TAG_MARKERREF)
        if markers:
            for m in markers:
                mre = MarkerRefElement(m, self.getOwnerWorkbook())
                # look for a marker of same familly
                if mre.getMarkerId().getFamilly() == markerId.getFamilly():
                    mre.setMarkerId(markerId)
                    return mre
        # not found so let's append it
        mre = MarkerRefElement(None, self.getOwnerWorkbook())
        mre.setMarkerId(markerId)
        tmp.appendChild(mre)
        return mre

    def setFolded(self):
        self.setAttribute(const.ATTR_BRANCH, const.VAL_FOLDED)

        # self.updateModifiedTime()

    def getPosition(self):
        """ Get a pair of integer located topic position.

        return (x, y) indicate x and y
        """
        position = self._get_position()
        if position is None:
            return

        position = PositionElement(position, self.getOwnerWorkbook())

        x = position.getX()
        y = position.getY()

        if x is None and y is None:
            return

        x = x or 0
        y = y or 0

        return (int(x), int(y))

    def setPosition(self, x, y):
        ownerWorkbook = self.getOwnerWorkbook()
        position = self._get_position()

        if not position:
            position = PositionElement(ownerWorkbook=ownerWorkbook)
            self.appendChild(position)
        else:
            position = PositionElement(position, ownerWorkbook)

        position.setX(x)
        position.setY(y)

        # self.updateModifiedTime()

    def removePosition(self):
        position = self._get_position()
        if position is not None:
            self.getImplementation().removeChild(position)

        # self.updateModifiedTime()

    def getType(self):
        parent = self.getParentNode()
        if not parent:
            return

        if parent.tagName == const.TAG_SHEET:
            return const.TOPIC_ROOT

        if parent.tagName == const.TAG_TOPICS:
            topics = TopicsElement(parent, self.getOwnerWorkbook())
            return topics.getType()

    def getTopics(self, topics_type=const.TOPIC_ATTACHED):
        topic_children = self._get_children()

        if topic_children:
            topic_children = ChildrenElement(
                topic_children,
                self.getOwnerWorkbook())

            return topic_children.getTopics(topics_type)

    def getSubTopics(self, topics_type=const.TOPIC_ATTACHED):
        """ List all sub topics under current topic, If not sub topics,
        return None.
        """
        topics = self.getTopics(topics_type)
        if not topics:
            return

        return topics.getSubTopics()

    def getSubTopicByIndex(self, index, topics_type=const.TOPIC_ATTACHED):
        """ Get sub topic by speicifeid index
        """
        sub_topics = self.getSubTopics(topics_type)
        if sub_topics is None:
            return

        if index < 0 or index >= len(sub_topics):
            return sub_topics

        return sub_topics[index]

    def addSubTopic(self, topic=None, index=-1,
                    topics_type=const.TOPIC_ATTACHED):
        """
        Add a sub topic to the current topic and return added sub topic

        :param topic:   `TopicElement` object. If not `TopicElement` object
                        passed then created new one automatically.
        :param index:   if index not given then passed topic will append to
                        sub topics list. Otherwise, index must be less than
                        length of sub topics list and insert passed topic
                        before given index.
        """
        ownerWorkbook = self.getOwnerWorkbook()
        topic = topic or self.__class__(None, ownerWorkbook)

        topic_children = self._get_children()
        if not topic_children:
            topic_children = ChildrenElement(ownerWorkbook=ownerWorkbook)
            self.appendChild(topic_children)
        else:
            topic_children = ChildrenElement(topic_children, ownerWorkbook)

        topics = topic_children.getTopics(topics_type)
        if not topics:
            topics = TopicsElement(ownerWorkbook=ownerWorkbook)
            topics.setAttribute(const.ATTR_TYPE, topics_type)
            topic_children.appendChild(topics)

        topic_list = []
        for i in topics.getChildNodesByTagName(const.TAG_TOPIC):
            topic_list.append(TopicElement(i, ownerWorkbook))

        if index < 0 or len(topic_list) >= index:
            topics.appendChild(topic)
        else:
            topics.insertBefore(topic, topic_list[index])

        return topic

    def getIndex(self):
        parent = self.getParentNode()
        if parent and parent.tagName == const.TAG_TOPICS:
            index = 0
            for child in parent.childNodes:
                if self.getImplementation() == child:
                    return index
                index += 1
        return -1

    def getHyperlink(self):
        return self.getAttribute(const.ATTR_HREF)

    def setFileHyperlink(self, path):
        """
        Set file as topic hyperlink

        :param path: path of specified file

        """
        protocol, content = split_hyperlink(path)
        if not protocol:
            path = const.FILE_PROTOCOL + utils.get_abs_path(path)

        self._set_hyperlink(path)

    def setTopicHyperlink(self, tid):
        """
        Set topic as topic hyperlink

        :param id: given topic's id

        """
        protocol, content = split_hyperlink(tid)
        if not protocol:
            if tid.startswith("#"):
                tid = tid[1:]

            tid = const.TOPIC_PROTOCOL + tid
        self._set_hyperlink(tid)

    def setURLHyperlink(self, url):
        """ Set URL as topic hyperlink

        :param url: HTTP URL to specified website

        """
        protocol, content = split_hyperlink(url)
        if not protocol:
            url = const.HTTP_PROTOCOL + content

        self._set_hyperlink(url)

    def getNotes(self):
        """
        Return `NotesElement` object` and invoke
        `NotesElement.getContent()` to get notes content.
        """

        notes = self.getFirstChildNodeByTagName(const.TAG_NOTES)

        if notes is not None:
            return NotesElement(notes, self)

    def _set_notes(self):
        notes = self.getNotes()

        if notes is None:
            notes = NotesElement(ownerTopic=self)
            self.appendChild(notes)

        return notes

    def setPlainNotes(self, content):
        """ Set plain text notes to topic

        :param content: utf8 plain text

        """
        notes = self._set_notes()
        new = PlainNotes(content, None, self)

        old = notes.getFirstChildNodeByTagName(new.getFormat())
        if old is not None:
            notes.getImplementation().removeChild(old)

        notes.appendChild(new)


class ChildrenElement(WorkbookMixinElement):
    TAG_NAME = const.TAG_CHILDREN

    def __init__(self, node=None, ownerWorkbook=None):
        super(ChildrenElement, self).__init__(node, ownerWorkbook)

    def getTopics(self, topics_type):
        topics = self.iterChildNodesByTagName(const.TAG_TOPICS)
        for i in topics:
            t = TopicsElement(i, self.getOwnerWorkbook())
            if topics_type == t.getType():
                return t


class TopicsElement(WorkbookMixinElement):
    TAG_NAME = const.TAG_TOPICS

    def __init__(self, node=None, ownerWorkbook=None):
        super(TopicsElement, self).__init__(node, ownerWorkbook)

    def getType(self):
        return self.getAttribute(const.ATTR_TYPE)

    def getSubTopics(self):
        """
        List all sub topics on the current topic
        """
        topics = []
        ownerWorkbook = self.getOwnerWorkbook()
        for t in self.getChildNodesByTagName(const.TAG_TOPIC):
            topics.append(TopicElement(t, ownerWorkbook))

        return topics

    def getSubTopicByIndex(self, index):
        """
        Get specified sub topic by index
        """
        sub_topics = self.getSubTopics()
        if index < 0 or index >= len(sub_topics):
            return sub_topics

        return sub_topics[index]


def main():
    pass

if __name__ == '__main__':
    main()



# _*_ coding=utf-8 _*_
__author__  = "8034.com"
__date__    = "2018-05-03"

from setuptools import setup, find_packages
import sys
import os

# 如果是Windows，显示“nt”，如果是Linux，则“posix”
if os.name == u"nt":
    os.system("cd xxmind/file/xmind-sdk-python-master & python setup.py install & cd ../../..")
    # xmind_tar_gz = "http://qa.ymmoa.com/wrench/file/xmind-0.1a0.tar.gz"
else:
    os.system("cd xxmind/file/xmind-sdk-python-master && python setup.py install && cd ../../..")
    pass

from xxmind.__version__ import __VERSION__

setup(
    name="xxmind",
    version = __VERSION__,
    author="goblinintree",
    author_email="goblinintree@126.com",
    description="xxmind是将xmind固定格式转为xlsx的转换工具。",
    long_description=open("README.md",encoding="UTF-8").read(),
    long_description_content_type="text/markdown",
    platforms=["all"],
    license="MIT",
    keywords = ['xlsx', 'excel', 'xmind'],
    url="https://pypi.org/project/xxmind/#description",
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        "Environment :: Web Environment",
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft',
        'Programming Language :: Python :: 3.7',
    ],
    zip_safe=True,
    install_requires=[
        "xlrd",
        "xlutils",
        "xlwt",
        "openpyxl",
    ],
    dependency_links = [
    ],
)


#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind.core.sheet
    ~~~~~~~~~~~~~~~~

    :mod:``xmind.core.sheet` command XMind sheets manipulation

    :copytright:
    :license:
"""

__author__ = "aiqi@xmind.net <Woody Ai>"

from . import const

from .mixin import WorkbookMixinElement
from .topic import TopicElement
from .title import TitleElement
from .relationship import RelationshipElement, RelationshipsElement


class SheetElement(WorkbookMixinElement):
    TAG_NAME = const.TAG_SHEET

    def __init__(self, node=None, ownerWorkbook=None):
        super(SheetElement, self).__init__(node, ownerWorkbook)

        self.addIdAttribute(const.ATTR_ID)
        self._root_topic = self._get_root_topic()

    def _get_root_topic(self):
        # This method initialize root topic, if not root topic
        # DOM implementation, then create one
        topics = self.getChildNodesByTagName(const.TAG_TOPIC)
        owner_workbook = self.getOwnerWorkbook()
        if len(topics) >= 1:
            root_topic = topics[0]
            root_topic = TopicElement(root_topic, owner_workbook)
        else:
            root_topic = TopicElement(ownerWorkbook=owner_workbook)
            self.appendChild(root_topic)

        return root_topic

    def createRelationship(self, end1, end2, title=None):
        """
        Create a relationship between two different topics and return the
        created rel. Please notice that the created rel will not be added to
        sheet. Call `addRelationship()` to add rel to sheet.

        :param end1:    topic ID
        :param end2:    topic ID
        :param title:   relationship title, default by None

        """
        rel = RelationshipElement(ownerWorkbook=self.getOwnerWorkbook())
        rel.setEnd1ID(end1)
        rel.setEnd2ID(end2)

        if title is not None:
            rel.setTitle(title)

        return rel

    def _getRelationships(self):
        return self.getFirstChildNodeByTagName(const.TAG_RELATIONSHIPS)

    def addRelationship(self, rel):
        """
        Add relationship to sheet
        """
        _rels = self._getRelationships()
        owner_workbook = self.getOwnerWorkbook()

        rels = RelationshipsElement(_rels, owner_workbook)

        if not _rels:
            self.appendChild(rels)

        rels.appendChild(rel)

    def removeRelationship(self, rel):
        """
        Remove a relationship between two different topics
        """
        rels = self._getRelationships()

        if not rels:
            return

        rel = rel.getImplementation()
        rels.removeChild(rel)
        if not rels.hasChildNodes():
            self.getImplementation().removeChild(rels)

        self.updateModifiedTime()

    def getRootTopic(self):
        return self._root_topic

    def _get_title(self):
        return self.getFirstChildNodeByTagName(const.TAG_TITLE)

    # FIXME: convert to getter/setter
    def getTitle(self):
        title = self._get_title()
        if title:
            title = TitleElement(title, self.getOwnerWorkbook())
            return title.getTextContent()

    def setTitle(self, text):
        _title = self._get_title()
        title = TitleElement(_title, self.getOwnerWorkbook())
        title.setTextContent(text)

        if _title is None:
            self.appendChild(title)

        self.updateModifiedTime()

    def getParent(self):
        workbook = self.getOwnerWorkbook()
        if workbook:
            parent = self.getParentNode()

            if (parent == workbook.getWorkbookElement().getImplementation()):
                return workbook

    def updateModifiedTime(self):
        super(SheetElement, self).updateModifiedTime()

        workbook = self.getParent()
        if workbook:
            workbook.updateModifiedTime()


def main():
    pass


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
    xmind.mixin
    ~~~~~~~~~~~

    :copyright:
    :license:

"""

__author__ = "aiqi@xmind.net <Woody Ai>"

from . import const
from . import Element

from .. import utils


class WorkbookMixinElement(Element):
    """
    """
    def __init__(self, node=None, ownerWorkbook=None):
        super(WorkbookMixinElement, self).__init__(node)
        self._owner_workbook = ownerWorkbook
        self.registOwnerWorkbook()

    def registOwnerWorkbook(self):
        if self._owner_workbook:
            self.setOwnerDocument(self._owner_workbook.getOwnerDocument())

    def getOwnerWorkbook(self):
        return self._owner_workbook

    def setOwnerWorkbook(self, workbook):
        if not self._owner_workbook:
            self._owner_workbook = workbook

    def getModifiedTime(self):
        timestamp = self.getAttribute(const.ATTR_TIMESTAMP)
        if timestamp:
            return utils.readable_time(timestamp)

    def setModifiedTime(self, time):
        self.setAttribute(const.ATTR_TIMESTAMP, long(time))

    def updateModifiedTime(self):
        self.setModifiedTime(utils.get_current_time())

    def getID(self):
        return self.getAttribute(const.ATTR_ID)


class TopicMixinElement(Element):
    def __init__(self, node, ownerTopic):
        super(TopicMixinElement, self).__init__(node)
        self._owner_topic = ownerTopic

    def getOwnerTopic(self):
        return self._owner_topic

    def getOwnerSheet(self):
        if not self._owner_topic:
            return

        return self._owner_topic.getOwnerSheet()

    def getOwnerWorkbook(self):
        if not self._owner_topic:
            return

        return self._owner_topic.getOwnerWorkbook()


def main():
    pass

if __name__ == "__main__":
    main()
